package com.jpattern.orm;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jpattern.orm.cache.CacheManager;
import com.jpattern.orm.exception.OrmConfigurationException;
import com.jpattern.orm.mapper.OrmClassTool;
import com.jpattern.orm.mapper.OrmClassToolImpl;
import com.jpattern.orm.mapper.ServiceCatalog;
import com.jpattern.orm.mapper.ServiceCatalogImpl;
import com.jpattern.orm.mapper.clazz.ClassMapBuilderImpl;
import com.jpattern.orm.mapper.clazz.ClassMap;
import com.jpattern.orm.persistor.OrmPersistor;
import com.jpattern.orm.persistor.PersistorGeneratorImpl;
import com.jpattern.orm.persistor.type.ExtendedTypeWrapper;
import com.jpattern.orm.persistor.type.TypeFactory;
import com.jpattern.orm.session.NullSessionProvider;
import com.jpattern.orm.session.SessionImpl;
import com.jpattern.orm.session.SessionProvider;
import com.jpattern.orm.validator.ValidatorService;

/**
 * 
 * @author Francesco Cina'
 *
 * 26/ago/2011
 */
public class JPOrm implements JPO {

    private static Integer JPORM_INSTANCES_COUNT = Integer.valueOf(0);
    private SessionImpl session;
    private ServiceCatalogImpl serviceCatalog = new ServiceCatalogImpl(this);
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    private final Integer instanceCount;

    /**
     * Create a new instance of JPOrm.
     * 
     * @param sessionProvider
     */
    public JPOrm(final SessionProvider sessionProvider) {
        synchronized (JPORM_INSTANCES_COUNT) {
            instanceCount = JPORM_INSTANCES_COUNT++;
        }
        logger.info("Building new instance of JPO (instance [{}])", instanceCount); //$NON-NLS-1$
        serviceCatalog.setDbProfile(sessionProvider.getDBType().getDBProfile());
        session = new SessionImpl(serviceCatalog, sessionProvider);
    }

    @Override
    public final SessionImpl session() {
        return session;
    }

    @Override
    public synchronized <BEAN> void register(final Class<BEAN> clazz) throws OrmConfigurationException {
        try {
            if (!getServiceCatalog().containsTool(clazz)) {
                logger.debug("register new class: " + clazz.getName()); //$NON-NLS-1$
                final ClassMap<BEAN> classMap = new ClassMapBuilderImpl<BEAN>(clazz, getServiceCatalog()).generate();
                final OrmPersistor<BEAN> ormPersistor =  new PersistorGeneratorImpl<BEAN>(getServiceCatalog(), classMap, getTypeFactory()).generate();
                final OrmClassTool<BEAN> ormClassTool = new OrmClassToolImpl<BEAN>(classMap, ormPersistor);
                serviceCatalog.put(clazz, ormClassTool);
            }
        } catch (final Exception e) {
            throw new OrmConfigurationException(e);
        }
    }

    @Override
    public synchronized void destory() {
        serviceCatalog = new ServiceCatalogImpl(this);
        session = new SessionImpl(serviceCatalog, new NullSessionProvider());
    }

    @Override
    public synchronized void register(final List<Class<?>> classes) throws OrmConfigurationException {
        for (final Class<?> clazz : classes) {
            this.register(clazz);
        }
    }

    @Override
    public synchronized void register(final ExtendedTypeWrapper<?, ?> typeWrapper) throws OrmConfigurationException {
        getTypeFactory().addTypeWrapper(typeWrapper);
    }

    @Override
    public synchronized void setValidatorService(final ValidatorService validatorService) {
        if (validatorService!=null) {
            serviceCatalog.setValidatorService(validatorService);
        }
    }

    public TypeFactory getTypeFactory() {
        return getServiceCatalog().getTypeFactory();
    }

    public ServiceCatalog getServiceCatalog() {
        return serviceCatalog;
    }

    @Override
    public void setCacheManager(final CacheManager cacheManager) {
        if (cacheManager!=null) {
            serviceCatalog.setCacheManager(cacheManager);
        }
    }

}
