package com.jpattern.orm.query;

import java.math.BigDecimal;
import java.util.List;

import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.exception.OrmNotUniqueResultException;
import com.jpattern.orm.session.ResultSetReader;
import com.jpattern.orm.session.ResultSetRowReader;

/**
 * 
 * @author Francesco Cina
 *
 * 09/lug/2011
 */
public interface CustomQuery {

	/**
	 * Execute the query reading the ResultSet with a
	 * {@link ResultSetReader}.
	 * @param rse object that will extract all rows of results
	 * @return an arbitrary result object, as returned by the {@link ResultSetReader}
	 */
	<T> T get(ResultSetReader<T> rsr) throws OrmException;

	/**
	 * Execute the query reading the ResultSet with a
	 * {@link ResultSetRowReader}.
	 * @param rsrr object that will extract all rows of results
	 * @return a List of result objects returned by the {@link ResultSetRowReader}
	 */
	<T> List<T> get(ResultSetRowReader<T> rsrr) throws OrmException;

	/**
	 * Execute the query reading the ResultSet with a
	 * {@link ResultSetRowReader}.
	 * @param rsrr object that will extract the row of result
	 * @return
	 * @throws OrmException
	 * @throws OrmNotUniqueResultException if the results of the query executions are not exactly 1
	 */
	<T> T getUnique(ResultSetRowReader<T> rsrr) throws OrmException, OrmNotUniqueResultException;

	/**
	 * Execute the query and read the result creating a List of all the ordered arrays with the extracted column values for every row.
	 * @return
	 */
	List<Object[]> getList() throws OrmException;

	/**
	 * Execute the query and read the result creating an ordered array with the extracted column values or null (if no matching bean is found).
	 * @return
	 * @throws OrmNotUniqueResultException if not exactly one row is returned from the query
	 */
	Object[] getUnique() throws OrmNotUniqueResultException;

	/**
	 * Execute the query and read the result as an int value
	 * @param sql SQL query to execute
	 * @param args arguments to bind to the query
	 * @return
	 */
	int getInt() throws OrmException;

	/**
	 * Execute the query and read the result as an long value
	 * @param sql SQL query to execute
	 * @param args arguments to bind to the query
	 * @return
	 */
	long getLong() throws OrmException;

	/**
	 * Execute the query and read the result as a double value
	 * @param sql SQL query to execute
	 * @param args arguments to bind to the query
	 * @return
	 */
	double getDouble() throws OrmException;

	/**
	 * Execute the query and read the result as a float value
	 * @param sql SQL query to execute
	 * @param args arguments to bind to the query
	 * @return
	 */
	float getFloat() throws OrmException;

	/**
	 * Execute the query and read the result as a String value
	 * @param sql SQL query to execute
	 * @param args arguments to bind to the query
	 * @return
	 */
	String getString() throws OrmException;

	/**
	 * Execute the query and read the result as a boolean value
	 * @param sql SQL query to execute
	 * @param args arguments to bind to the query
	 * @return
	 */
	boolean getBoolean() throws OrmException;

	/**
	 * Execute the query and read the result as a BigDecimal value
	 * @param sql SQL query to execute
	 * @param args arguments to bind to the query
	 * @return
	 */
	BigDecimal getBigDecimal() throws OrmException;

}
