package com.jpattern.orm.query.delete;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import com.jpattern.orm.mapper.OrmClassTool;
import com.jpattern.orm.mapper.ServiceCatalog;
import com.jpattern.orm.mapper.clazz.ClassFieldImpl;
import com.jpattern.orm.mapper.clazz.ClassMap;
import com.jpattern.orm.mapper.relation.RelationInnerFK;
import com.jpattern.orm.mapper.relation.RelationOuterFK;
import com.jpattern.orm.query.SmartRenderableSqlQuery;
import com.jpattern.orm.query.clause.where.ExpressionElement;
import com.jpattern.orm.query.find.CustomFindQuery;
import com.jpattern.orm.query.namesolver.NameSolver;
import com.jpattern.orm.query.namesolver.NameSolverImpl;
import com.jpattern.orm.session.ResultSetRowReader;
import com.jpattern.orm.session.SessionImpl;
import com.jpattern.orm.session.SqlPerformer;
import com.jpattern.orm.util.ObjectBuilder;

/**
 * 
 * @author Francesco Cina
 *
 * 10/lug/2011
 */
public class DeleteQueryOrm<BEAN> extends SmartRenderableSqlQuery implements DeleteQuery<BEAN> {

    private boolean _cascade = false;
    private final DeleteWhereImpl<BEAN> where = new DeleteWhereImpl<BEAN>(this);
    private final ServiceCatalog serviceCatalog;
    private final SessionImpl session;
    private int _queryTimeout = 0;
    private final Class<?> clazz;
    private final NameSolver nameSolver;
    private OrmClassTool<BEAN> ormClassTool;

    public DeleteQueryOrm(final Class<BEAN> clazz, final ServiceCatalog serviceCatalog, final SessionImpl session) {
        this.clazz = clazz;
        this.serviceCatalog = serviceCatalog;
        this.session = session;
        nameSolver = new NameSolverImpl(serviceCatalog, true);
        nameSolver.register(clazz, clazz.getSimpleName());
        ormClassTool = serviceCatalog.getOrmClassTool(clazz);
    }

    @Override
    public DeleteWhere<BEAN> where() {
        return where;
    }

    @Override
    public int now() {
        int deleted = deleteOuterRelations();
        Map<String, Object> innerRelationValues = fetchInnerRelationValue();

        final List<Object> values = new ArrayList<Object>();
        appendValues(values);
        final SqlPerformer sqlExec = session.sqlPerformer();
        sqlExec.setQueryTimeout(getQueryTimeout());
        deleted+= sqlExec.update(renderSql(), values);

        deleted+= deleteInnerRelations(innerRelationValues);
        return deleted;
    }

    private int deleteOuterRelations() {
        int deleted = 0;
        if (_cascade) {
            List<RelationOuterFK<BEAN, ?, ?>> relations = ormClassTool.getClassMap().getOuterRelations();

            for (RelationOuterFK<BEAN, ?, ?> relation : relations) {
                if (relation.getCascadeInfo().onDelete()) {
                    Class<?> relationWith = relation.getRelationWithClass();
                    String relationFieldName = serviceCatalog.getOrmClassTool(relationWith).getClassMap().getFKs().versus(clazz).getBeanFieldName();

                    String alias = clazz.getSimpleName();
                    final String[] pkName = ormClassTool.getClassMap().getPrimaryKeyColumnJavaNames();
                    CustomFindQuery idQuery = session.findQuery(pkName, clazz, alias);
                    for (ExpressionElement expressionElement : where.getElementList()) {
                        idQuery.where().and(expressionElement);
                    }
                    //                List<Object> ids = idQuery.find(new ResultSetRowReader<Object>() {
                    //                    @Override
                    //                    public Object readRow(final ResultSet rs, final int rowNum) throws SQLException {
                    //                        return rs.getObject(pkName);
                    //                    }
                    //                });
                    deleted+=session.deleteQuery(relationWith).cascade(_cascade).where().in(relationFieldName, idQuery).now();
                }
            }
        }
        return deleted;
    }


    private Map<String, Object> fetchInnerRelationValue() {
        final Map<String, Object> values = new HashMap<String, Object>();
        final List<RelationInnerFK<BEAN, ?>> allRelations = ormClassTool.getClassMap().getInnerRelations();
        final List<String> relationFieldNames = new ArrayList<String>();

        for (RelationInnerFK<BEAN, ?> relationInnerFK : allRelations) {
            if (relationInnerFK.getCascadeInfo().onDelete()) {
                relationFieldNames.add(relationInnerFK.getFieldName());
            }
        }
        if (_cascade && !relationFieldNames.isEmpty()) {
            String alias = clazz.getSimpleName();
            ResultSetRowReader<Void> rsrr = new ResultSetRowReader<Void>() {
                @Override
                public Void readRow(final ResultSet rs, final int rowNum) throws SQLException {
                    for (String relationFieldName : relationFieldNames) {
                        values.put(relationFieldName, rs.getObject(relationFieldName) );
                    }
                    return null;
                }
            };
            CustomFindQuery find = session.findQuery(relationFieldNames.toArray(ObjectBuilder.EMPTY_STRING_ARRAY), clazz, alias);
            for (ExpressionElement expressionElement : where.getElementList()) {
                find.where().and(expressionElement);
            }
            find.get(rsrr);
        }
        return values;
    }

    /**
     * @return
     */
    private <RELATION_WITH> int deleteInnerRelations(final Map<String, Object> innerRelationValues) {
        int deleted = 0;
        if (_cascade) {
            for (Entry<String, Object> innerRelationEntry : innerRelationValues.entrySet()) {
                Object fieldValue = innerRelationEntry.getValue();
                if (fieldValue!=null) {
                    String fieldName = innerRelationEntry.getKey();
                    ClassFieldImpl<BEAN, Object> field = ormClassTool.getClassMap().getClassFieldByJavaName(fieldName);
                    Class<RELATION_WITH> relationWith = field.getRelationVersusClass();
                    ClassMap<RELATION_WITH> relationWithClassMap = serviceCatalog.getOrmClassTool(relationWith).getClassMap();
                    String relationWithPkFieldName = relationWithClassMap.getPrimaryKeyColumnJavaNames()[0];
                    deleted+=session.deleteQuery(relationWith).cascade(_cascade).where().eq(relationWithPkFieldName, fieldValue).now();
                }
            }
        }
        return deleted;
    }

    @Override
    public final void appendValues(final List<Object> values) {
        where.appendElementValues(values);
    }

    @Override
    public DeleteQuery<BEAN> queryTimeout(final int queryTimeout) {
        this._queryTimeout = queryTimeout;
        return this;
    }

    public int getQueryTimeout() {
        return _queryTimeout;
    }

    @Override
    public int getStatusVersion() {
        return where.getElementStatusVersion();
    }

    @Override
    public final void doRender(final StringBuilder queryBuilder) {
        queryBuilder.append("DELETE FROM "); //$NON-NLS-1$
        queryBuilder.append(serviceCatalog.getOrmClassTool(clazz).getClassMap().getTableInfo().getTableNameWithSchema() );
        queryBuilder.append(" "); //$NON-NLS-1$
        where.renderSqlElement(queryBuilder, nameSolver);
    }

    @Override
    public DeleteQuery<BEAN> cascade(final boolean cascade) {
        _cascade = cascade;
        return this;
    }

}
