package com.jpattern.orm.query.save;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Map;
import java.util.Map.Entry;

import com.jpattern.orm.exception.OrmReflectionException;
import com.jpattern.orm.mapper.OrmClassTool;
import com.jpattern.orm.mapper.ServiceCatalog;
import com.jpattern.orm.mapper.clazz.ClassField;
import com.jpattern.orm.mapper.clazz.ClassMap;
import com.jpattern.orm.persistor.OrmPersistor;
import com.jpattern.orm.query.saveorupdate.ASaveOrUpdateQueryOrm;
import com.jpattern.orm.query.saveorupdate.SaveOrUpdateType;
import com.jpattern.orm.session.GeneratedKeyReader;
import com.jpattern.orm.session.SessionImpl;
import com.jpattern.orm.session.SqlPerformer;

/**
 * 
 * @author Francesco Cina
 *
 * 10/lug/2011
 */
public class SaveQueryOrm<BEAN> extends ASaveOrUpdateQueryOrm<BEAN> implements SaveQuery<BEAN> {

    private final SessionImpl session;
    private int queryTimeout = 0;
    private final Class<BEAN> clazz;
    private final OrmClassTool<BEAN> ormClassTool;
    private final BEAN bean;
    private boolean useGenerator = false;
    private boolean _cascade = false;
    private SaveOrUpdateType _saveOrUpdateType = SaveOrUpdateType.SAVE;
    private final ServiceCatalog serviceCatalog;

    public SaveQueryOrm(final BEAN bean, final ServiceCatalog serviceCatalog, final SessionImpl session) {
        this.bean = bean;
        this.serviceCatalog = serviceCatalog;
        this.clazz = (Class<BEAN>) bean.getClass();
        this.ormClassTool = serviceCatalog.getOrmClassTool(clazz);
        this.session = session;
    }

    @Override
    public BEAN now() {
        saveOrUpdateInnerRelations(_cascade, bean, ormClassTool, session, _saveOrUpdateType);

        final OrmPersistor<BEAN> persistor = ormClassTool.getOrmPersistor();
        final SqlPerformer sqlExec = this.session.sqlPerformer();
        sqlExec.setQueryTimeout(queryTimeout);

        //CHECK IF OBJECT HAS A 'VERSION' FIELD and increase it
        persistor.increaseVersion(bean, true);
        String sql = renderSql();
        if (!useGenerator) {
            String[] keys = ormClassTool.getClassMap().getAllColumnJavaNames();
            Object[] values = persistor.getPropertyValues(keys, bean);
            sqlExec.update(sql, values);
        } else {
            final GeneratedKeyReader generatedKeyExtractor = new GeneratedKeyReader() {

                @Override
                public void read(final ResultSet generatedKeyResultSet) throws SQLException {
                    if (generatedKeyResultSet.next()) {
                        persistor.updateGeneratedValues(generatedKeyResultSet, bean);
                    }
                }

                @Override
                public String[] generatedColumnNames() {
                    return ormClassTool.getClassMap().getAllGeneratedColumnDBNames();
                }
            };
            String[] keys = ormClassTool.getClassMap().getAllNotGeneratedColumnJavaNames();
            Object[] values = persistor.getPropertyValues(keys, bean);
            sqlExec.update(sql, generatedKeyExtractor, values);
        }
        saveOrUpdateOuterRelations(_cascade, bean, ormClassTool, session, _saveOrUpdateType);
        return bean;

    }

    @Override
    public SaveQuery<BEAN> setQueryTimeout(final int queryTimeout) {
        this.queryTimeout = queryTimeout;
        return this;
    }

    @Override
    public int getQueryTimeout() {
        return this.queryTimeout;
    }

    @Override
    public SaveQuery<BEAN> value(final String propertyName, final Object propertyValue) {
        try {
            ormClassTool.getClassMap().getClassFieldByJavaName(propertyName).getSetManipulator().setValue(bean, propertyValue);
            return this;
        } catch (Exception e) {
            throw new OrmReflectionException(e);
        }
    }

    @Override
    public SaveQuery<BEAN> values(final Map<String, Object> properties) {
        for (Entry<String, Object> propertyEntry : properties.entrySet()) {
            value(propertyEntry.getKey(), propertyEntry.getValue());
        }
        return this;
    }

    @SuppressWarnings("nls")
    public final String renderSql() {
        updateUseGenerators();
        return generateSaveQuery(!useGenerator); //$NON-NLS-1$
    }

    private String generateSaveQuery(final boolean ignoreGenerators) {
        ClassMap<BEAN> classMap = ormClassTool.getClassMap();
        final StringBuilder builder = new StringBuilder("INSERT INTO "); //$NON-NLS-1$
        builder.append(ormClassTool.getClassMap().getTableInfo().getTableNameWithSchema());
        builder.append(" ("); //$NON-NLS-1$
        builder.append( this.columnToCommaSepareted( "" , classMap.getAllColumnJavaNames(), ignoreGenerators )); //$NON-NLS-1$
        builder.append(") VALUES ("); //$NON-NLS-1$
        builder.append( this.questionCommaSepareted( classMap.getAllColumnJavaNames(), ignoreGenerators ));
        builder.append(") "); //$NON-NLS-1$
        return builder.toString();
    }

    private void updateUseGenerators() {
        useGenerator = ormClassTool.getOrmPersistor().useGenerators(bean);
    }

    private String questionCommaSepareted(final String[] fieldNames, final boolean ignoreGenerators) {
        final StringBuilder builder = new StringBuilder();
        ClassMap<BEAN> classMap = ormClassTool.getClassMap();
        final int length = fieldNames.length;
        boolean generatedKey = false;
        if (length > 0) {
            for (int i=0; i<(length-1) ; i++) {
                ClassField<BEAN, ?> classField = classMap.getClassFieldByJavaName(fieldNames[i]);
                final AColumnValueGenerator columnValueGenerator = ColumnValueGeneratorFactory.getColumnValueGenerator( classField, serviceCatalog.getDbProfile(), ignoreGenerators );
                generatedKey = generatedKey || columnValueGenerator.isAutoGenerated();
                final String queryParameter = columnValueGenerator.insertQueryParameter( "?"); //$NON-NLS-1$
                if (queryParameter.length()>0) {
                    builder.append( queryParameter );
                    builder.append(", "); //$NON-NLS-1$
                }
            }
            ClassField<BEAN, ?> classField = classMap.getClassFieldByJavaName(fieldNames[fieldNames.length-1]);
            final AColumnValueGenerator columnValueGenerator = ColumnValueGeneratorFactory.getColumnValueGenerator( classField, serviceCatalog.getDbProfile(), ignoreGenerators );
            generatedKey = generatedKey || columnValueGenerator.isAutoGenerated();
            builder.append( columnValueGenerator.insertQueryParameter( "?") ); //$NON-NLS-1$
        }
        return builder.toString();
    }

    private String columnToCommaSepareted(final String prefix, final String[] fieldNames, final boolean ignoreGenerators) {
        final StringBuilder builder = new StringBuilder();
        ClassMap<BEAN> classMap = ormClassTool.getClassMap();
        if (fieldNames.length > 0) {
            for (int i=0; i<(fieldNames.length-1) ; i++) {
                ClassField<BEAN, ?> classField = classMap.getClassFieldByJavaName(fieldNames[i]);
                final AColumnValueGenerator columnValueGenerator = ColumnValueGeneratorFactory.getColumnValueGenerator( classField, serviceCatalog.getDbProfile(), ignoreGenerators );
                final String queryParameter = columnValueGenerator.insertColumn(prefix + classField.getColumnInfo().getDBColumnName());
                if (queryParameter.length()>0) {
                    builder.append( queryParameter );
                    builder.append(", "); //$NON-NLS-1$
                }
            }
            ClassField<BEAN, ?> classField = classMap.getClassFieldByJavaName(fieldNames[fieldNames.length-1]);
            final AColumnValueGenerator columnValueGenerator = ColumnValueGeneratorFactory.getColumnValueGenerator( classField, serviceCatalog.getDbProfile(), ignoreGenerators );
            builder.append( columnValueGenerator.insertColumn( prefix + classField.getColumnInfo().getDBColumnName()) );
        }

        return builder.toString();
    }

    /**
     * @param cascade
     * @return
     */
    @Override
    public SaveQuery<BEAN> cascade(final boolean cascade) {
        this._cascade = cascade;
        return this;
    }

    @Override
    public SaveQuery<BEAN> saveOrUpdate(final SaveOrUpdateType saveOrUpdateType) {
        _saveOrUpdateType = saveOrUpdateType;
        return this;
    }

}
