package com.jpattern.orm.session;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.exception.OrmNotUniqueResultException;
import com.jpattern.orm.persistor.type.TypeFactory;
import com.jpattern.orm.session.reader.ArrayResultSetReader;
import com.jpattern.orm.session.reader.BigDecimalResultSetReader;
import com.jpattern.orm.session.reader.BooleanResultSetReader;
import com.jpattern.orm.session.reader.DoubleResultSetReader;
import com.jpattern.orm.session.reader.FloatResultSetReader;
import com.jpattern.orm.session.reader.IntegerResultSetReader;
import com.jpattern.orm.session.reader.ListResultSetReader;
import com.jpattern.orm.session.reader.LongResultSetReader;
import com.jpattern.orm.session.reader.ResultSetRowReaderToResultSetReader;
import com.jpattern.orm.session.reader.ResultSetRowReaderToResultSetReaderUnique;
import com.jpattern.orm.session.reader.StringResultSetReader;

/**
 * 
 * @author Francesco Cina
 *
 * 02/lug/2011
 */
public class PlainSqlPerformer implements SqlPerformer {

    private final SqlPerformerStrategy sqlPerformerStrategy;
    private final TypeFactory typeFactory;
    private int queryTimeout = 0;
    private int maxRows = 0;

    public PlainSqlPerformer(final SqlPerformerStrategy sqlPerformerStrategy, final TypeFactory typeFactory) {
        this.sqlPerformerStrategy = sqlPerformerStrategy;
        this.typeFactory = typeFactory;
    }

    @Override
    public final void setMaxRows(final int maxRows) {
        this.maxRows = maxRows;
    }

    @Override
    public final int getMaxRows() {
        return maxRows;
    }

    @Override
    public final void setQueryTimeout(final int queryTimeout) {
        this.queryTimeout = queryTimeout;
    }

    @Override
    public final int getQueryTimeout() {
        return queryTimeout;
    }

    @Override
    public final Integer queryForInt(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new IntegerResultSetReader(), values);
    }

    @Override
    public final Integer queryForInt(final String sql, final Collection<?> values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new IntegerResultSetReader(), values);
    }

    @Override
    public final Long queryForLong(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new LongResultSetReader(), values);
    }

    @Override
    public final Long queryForLong(final String sql, final Collection<?> values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new LongResultSetReader(), values);
    }

    @Override
    public final Double queryForDouble(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new DoubleResultSetReader(), values);
    }

    @Override
    public final Double queryForDouble(final String sql, final Collection<?> values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new DoubleResultSetReader(), values);
    }

    @Override
    public final Float queryForFloat(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new FloatResultSetReader(), values);
    }

    @Override
    public final Float queryForFloat(final String sql, final Collection<?> values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new FloatResultSetReader(), values);
    }

    @Override
    public final String queryForString(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new StringResultSetReader(), values);
    }

    @Override
    public final String queryForString(final String sql, final Collection<?> values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new StringResultSetReader(), values);
    }

    @Override
    public final Boolean queryForBoolean(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new BooleanResultSetReader(), values);
    }

    @Override
    public final Boolean queryForBoolean(final String sql, final Collection<?> values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new BooleanResultSetReader(), values);
    }

    @Override
    public final BigDecimal queryForBigDecimal(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new BigDecimalResultSetReader(), values);
    }

    @Override
    public final BigDecimal queryForBigDecimal(final String sql, final Collection<?> values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new BigDecimalResultSetReader(), values);
    }

    @Override
    public final Object[] queryForArray(final String sql, final Object... values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new ArrayResultSetReader(), values);
    }

    @Override
    public final Object[] queryForArray(final String sql, final Collection<?> values) throws OrmException, OrmNotUniqueResultException {
        return this.query(sql, new ArrayResultSetReader(), values);
    }

    @Override
    public final List<Object[]> queryForList(final String sql, final Object... values) throws OrmException {
        return this.query(sql, new ListResultSetReader(), values);
    }

    @Override
    public final List<Object[]> queryForList(final String sql, final Collection<?> values) throws OrmException {
        return this.query(sql, new ListResultSetReader(), values);
    }

    @Override
    public void execute(final String sql) throws OrmException {
        sqlPerformerStrategy.execute(sql, getQueryTimeout());
    }

    @Override
    public <T> T query(final String sql, final ResultSetReader<T> rse, final Object... args) throws OrmException {
        return sqlPerformerStrategy.query(sql, rse, getQueryTimeout(), getMaxRows(), args, typeFactory);
    }

    @Override
    public <T> T query(final String sql, final ResultSetReader<T> rse, final Collection<?> args) throws OrmException {
        return sqlPerformerStrategy.query(sql, rse, getQueryTimeout(), getMaxRows(), args, typeFactory);
    }

    @Override
    public <T> List<T> query(final String sql, final ResultSetRowReader<T> rsrr, final Object... args) throws OrmException {
        return query(sql, new ResultSetRowReaderToResultSetReader<T>(rsrr), args);
    }

    @Override
    public <T> List<T> query(final String sql, final ResultSetRowReader<T> rsrr, final Collection<?> args) throws OrmException {
        return query(sql, new ResultSetRowReaderToResultSetReader<T>(rsrr), args);
    }

    @Override
    public <T> T queryForUnique(final String sql, final ResultSetRowReader<T> rsrr, final Object... args) throws OrmException, OrmNotUniqueResultException {
        return query(sql, new ResultSetRowReaderToResultSetReaderUnique<T>(rsrr), args);
    }

    @Override
    public <T> T queryForUnique(final String sql, final ResultSetRowReader<T> rsrr, final Collection<?> args) throws OrmException {
        return query(sql, new ResultSetRowReaderToResultSetReaderUnique<T>(rsrr), args);
    }

    @Override
    public int update(final String sql, final Object... args) throws OrmException {
        return sqlPerformerStrategy.update(sql, getQueryTimeout(), args, typeFactory);
    }

    @Override
    public int update(final String sql, final Collection<?> args) throws OrmException {
        return sqlPerformerStrategy.update(sql, getQueryTimeout(), args, typeFactory);
    }

    @Override
    public int update(final String sql, final PreparedStatementSetter psc) throws OrmException {
        return sqlPerformerStrategy.update(sql, getQueryTimeout(), psc);
    }

    @Override
    public int update(final String sql, final GeneratedKeyReader generatedKeyReader, final Object... args) throws OrmException {
        return sqlPerformerStrategy.update(sql, getQueryTimeout(), generatedKeyReader, args, typeFactory);
    }

    @Override
    public int update(final String sql, final GeneratedKeyReader generatedKeyReader, final Collection<?> args) throws OrmException {
        return sqlPerformerStrategy.update(sql, getQueryTimeout(), generatedKeyReader, args, typeFactory);
    }

    @Override
    public int update(final String sql, final GeneratedKeyReader generatedKeyReader, final PreparedStatementSetter psc) throws OrmException {
        return sqlPerformerStrategy.update(sql, getQueryTimeout(), generatedKeyReader, psc);
    }

    @Override
    public int[] batchUpdate(final List<String> sqls) throws OrmException {
        return sqlPerformerStrategy.batchUpdate(sqls, getQueryTimeout());
    }

    @Override
    public int[] batchUpdate(final String sql, final List<Object[]> args) throws OrmException {
        return sqlPerformerStrategy.batchUpdate(sql, args, getQueryTimeout());
    }

    @Override
    public int[] batchUpdate(final String sql, final BatchPreparedStatementSetter psc) throws OrmException {
        return sqlPerformerStrategy.batchUpdate(sql, psc, getQueryTimeout());
    }

}
