package com.jpattern.orm.session.datasource;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Savepoint;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.exception.OrmRollbackException;

/**
 * 
 * <class_description>
 * <p><b>notes</b>:
 * <p>ON : Mar 9, 2013
 *
 * @author ARHS Developments - Francesco Cina
 * @version $Revision
 */
public class DataSourceConnectionImpl implements DataSourceConnection {

    private ConnectionWrapper connectionWrapper;
    private List<DataSourceConnectionCaller> connectionCallers = new ArrayList<DataSourceConnectionCaller>();
    private boolean rollbackOnly = false;
    private boolean readOnly = false;
    private boolean valid = true;

    public DataSourceConnectionImpl(final DataSource dataSource, final boolean readOnly) {
        connectionWrapper = new ConnectionWrapper(dataSource);
        setReadOnly(readOnly);
    }

    @Override
    public void setTransactionIsolation(final int transactionIsolation) throws OrmException {
        try {
            connectionWrapper.setTransactionIsolation(transactionIsolation);
        } catch (SQLException e) {
            throw new OrmException(e);
        }
    }

    @Override
    public boolean isClosed() throws OrmException {
        try {
            return connectionWrapper.isClosed();
        } catch (SQLException e) {
            throw new OrmException(e);
        }
    }

    @Override
    public void rollback() throws OrmException {
        if ((connectionCallers.size()==1) && !isReadOnly()) {
            try {
                connectionWrapper.rollback();
            } catch (SQLException e) {
                throw new OrmException(e);
            }
        }
    }

    @Override
    public void commit() throws OrmException {
        if ((connectionCallers.size()==1) && !isReadOnly()) {
            if (isRollbackOnly()) {
                rollback();
                throw new OrmRollbackException("Transaction rolled back because it has been marked as rollback-only"); //$NON-NLS-1$
            }
            try {
                connectionWrapper.commit();
            } catch (SQLException e) {
                throw new OrmException(e);
            }
        }
    }

    @Override
    public PreparedStatement prepareStatement(final String sql) throws OrmException {
        try {
            return connectionWrapper.prepareStatement(sql);
        } catch (SQLException e) {
            throw new OrmException(e);
        }
    }

    @Override
    public PreparedStatement prepareStatement(final String sql, final String[] generatedColumnNames) throws OrmException {
        try {
            return connectionWrapper.prepareStatement(sql, generatedColumnNames) ;
        } catch (SQLException e) {
            throw new OrmException(e);
        }
    }

    @Override
    public DataSourceStatement createStatement() throws OrmException {
        try {
            return new DataSourceStatementWrapper(connectionWrapper.createStatement() , this);
        } catch (SQLException e) {
            throw new OrmException(e);
        }
    }

    @Override
    public void addCaller(final DataSourceConnectionCaller connectionCaller) throws OrmException {
        connectionCallers.add(connectionCaller);
    }

    @Override
    public void close(final DataSourceConnectionCaller connectionCaller) throws OrmException {
        connectionCallers.remove(connectionCaller);
        if (connectionCallers.isEmpty()) {
            try {
                connectionWrapper.close();
                valid = false;
            } catch (SQLException e) {
                throw new OrmException(e);
            }
        }
    }

    @Override
    public void setRollbackOnly() throws OrmException {
        rollbackOnly = true;
    }

    @Override
    public void setReadOnly(final boolean readOnly) throws OrmException {
        this.readOnly = readOnly;
    }

    @Override
    public boolean isValid() throws OrmException {
        return valid;
    }

    public boolean isReadOnly() {
        return readOnly;
    }

    /**
     * @return the rollbackOnly
     */
    @Override
    public boolean isRollbackOnly() {
        return rollbackOnly;
    }

    class ConnectionWrapper {

        private java.sql.Connection connection;
        private final DataSource dataSource;
        private Savepoint savepoint;

        ConnectionWrapper(final DataSource dataSource) {
            this.dataSource = dataSource;
        }

        public void setTransactionIsolation(final int transactionIsolation) throws SQLException {
            validateConnection();
            connection.setTransactionIsolation(transactionIsolation);
        }

        public void rollback() throws SQLException {
            if (connection!=null) {
                if (savepoint==null) {
                    connection.rollback();
                } else {
                    connection.rollback(savepoint);
                }

            }
        }

        public void rollback(final Savepoint rollbackToSavepoint) throws SQLException {
            if (connection!=null) {
                connection.rollback(rollbackToSavepoint);
            }
        }

        public void commit() throws SQLException {
            if (connection!=null) {
                connection.commit();
            }
        }

        public Statement createStatement() throws SQLException {
            validateConnection();
            return connection.createStatement();
        }

        public PreparedStatement prepareStatement(final String sql) throws SQLException {
            validateConnection();
            return connection.prepareStatement(sql);
        }

        public PreparedStatement prepareStatement(final String sql, final String[] generatedColumnNames) throws SQLException {
            validateConnection();
            return connection.prepareStatement(sql, generatedColumnNames);
        }

        public void close() throws SQLException {
            if (connection!=null) {
                connection.close();
            }
        }

        public boolean isClosed() throws SQLException {
            if (connection!=null) {
                return connection.isClosed();
            }
            return true;
        }

        private void validateConnection() throws SQLException {
            if (!isValid()) {
                throw new OrmException("Not possible to open a new java.sql.Connection"); //$NON-NLS-1$
            }
            if (isClosed()) {
                Connection sqlConn = dataSource.getConnection();
                if (!isReadOnly()) {
                    savepoint = sqlConn.setSavepoint();
                }
                connection = sqlConn;
            }
        }

    }
}
